from typing import Any, Dict, Optional, Union
from uuid import uuid4

import confluent_kafka as ck
import orjson

from . import get_logger


class Producer:
    def __init__(
        self,
        host: str,
        port: Union[int, str],
    ) -> None:
        """
        Initialize the Producer instance.

        Args:
            host (str): The hostname or IP address of the Kafka broker.
            port (Union[int, str]): The port number of the Kafka broker.
            topic (str): The name of the topic to produce messages to.
        """
        self.p = ck.Producer({"bootstrap.servers": f"{host}:{port}"})
        self.log = get_logger(__name__)

    def delivery_report(self, err: Optional[ck.KafkaError], msg: ck.Message) -> None:
        """
        Reports the failure or success of a message delivery.

        Args:
            err (Optional[ck.KafkaError]): The error that occurred, or None on success.
            msg (ck.Message): The message that was produced or failed.

        Note:
            In the delivery report callback, the Message.key() and Message.value()
            will be in binary format as encoded by any configured Serializers and
            not the same object that was passed to produce().
            If you wish to pass the original object(s) for key and value to the delivery
            report callback, we recommend using a bound callback or lambda where you pass
            the objects along.
        """
        if err is not None:
            self.log.error(f"Delivery failed for User record {msg.key()}: {err}")
            return
        self.log.info(
            f"Record {msg.key()} successfully produced to {msg.topic()} [{msg.partition()}] at offset {msg.offset()}"
        )

    def send_json(self, message: Dict[Any, Any], topic: str) -> None:
        """
        Send a JSON message to the Kafka topic.

        Args:
            message (dict): The message to be sent, represented as a dictionary.
            topic (str): The name of the topic to send the message to.
        """
        self.p.produce(
            topic=topic,
            key=str(uuid4()),
            value=orjson.dumps(message),
            on_delivery=self.delivery_report,
        )
        self.p.flush()

    def send_binary(self, payload: bytes, topic: str) -> None:
        """
        Send a binary payload to the Kafka topic.

        Args:
            payload (bytes): The payload to be sent, represented as a bytestring.
            topic (str): The name of the topic to send the payload to.
        """
        self.p.produce(
            topic=topic,
            key=str(uuid4()),
            value=payload,
            on_delivery=self.delivery_report,
        )
        self.p.flush()



import datetime
import glob
import os
from typing import Any, Dict, Union

import astropy.io.fits as fits
import pandas as pd
import pymongoarrow.monkey
from pymongo import MongoClient
from pymongoarrow.api import Schema

from . import config, models
from .models import fitspath_to_constructor
from .producer import Producer

pymongoarrow.monkey.patch_all()


def images_containing(
    ra: float, dec: float, date_start: datetime.datetime, date_end: datetime.datetime
) -> pd.DataFrame:
    """
    Retrieve images containing a given position within a specified time range.

    Args:
        ra (float): Right ascension (ICRS) of the target position.
        dec (float): Declination (ICRS) of the target position.
        date_start (datetime.datetime): Start date of the time range.
        date_end (datetime.datetime): End date of the time range.

    Returns:
        pd.DataFrame: A DataFrame containing the retrieved images.

    """
    client = MongoClient(config.MONGODB_URI)
    collection = client[config.MONGO_DBNAME].evr_images
    images = collection.find_pandas_all(
        {
            "$and": [{"rawpath": {"$ne": None}}, {"wcspath": {"$ne": None}}],
            "footprint": {
                "$geoIntersects": {
                    "$geometry": {"type": "Point", "coordinates": [ra - 180.0, dec]}
                }
            },
            "obstime": {"$gte": date_start, "$lte": date_end},
            "image_type": "object",
        },
        schema=Schema(
            {
                "obstime": datetime.datetime,
                "image_type": str,
                "camera": str,
                "rawpath": str,
                "wcspath": str,
            }
        ),
    )
    return images


class EVRImageProducer(Producer):
    """
    A Kafka producer for sending EVR images.

    Args:
        None

    Attributes:
        host (str): The Kafka host address.
        port (int): The Kafka port number.

    """

    def __init__(self) -> None:
        """
        Initialize the EVRImageProducer.

        Args:
            None

        Returns:
            None

        """
        super().__init__(
            host=config.KAFKA_ADDR,
            port=config.KAFKA_PORT,
        )

    def send_image(self, image: Union[str, fits.HDUList]) -> None:
        """
        Send an image to the EVR image topic.

        Args:
            image (Union[str, fits.HDUList]): The image to be sent. It can be either a string representing the
                path to a FITS file or a `fits.HDUList` object.

        Returns:
            None

        """
        image_dict = fitspath_to_constructor(image)
        self.send_json(image_dict, config.EVR_IMAGE_TOPIC)


class EVRImageLoader:
    """Class for loading EVRImage data into a MongoDB collection."""

    def __init__(self, create_client: bool = True) -> None:
        """
        Initialize EVRImageLoader.

        Args:
            create_client (bool): Whether to create a MongoDB client. Default is True.

        Returns:
            None
        """
        self.client: MongoClient[Dict[str, Any]]
        self.client_loaded: bool = create_client

        if create_client:
            self.client = MongoClient(config.MONGODB_URI, uuidRepresentation="standard")

    def load_fits(self, path: str) -> None:
        """
        Load FITS file data into the MongoDB collection.

        Args:
            path (str): Path to the FITS file.

        Returns:
            None
        """
        img_coll = self.client[config.MONGO_DBNAME].evr_images

        db_img = img_coll.find_one({"basename": os.path.basename(path).split(".")[0]})
        if db_img is None:
            img_new = models.EVRImage.from_fits(path)
            _ = img_coll.insert_one(img_new.dict())
        else:
            img_existing = models.EVRImageUpdate.from_fits(path)
            _ = img_coll.update_one(
                {"_id": db_img["_id"]}, {"$set": img_existing.dict(exclude_unset=True)}
            )

    def load_directory(self, dirname: str) -> None:
        """
        Load all FITS files from a directory into the MongoDB collection.

        Args:
            dirname (str): Directory path containing FITS files.

        Returns:
            None
        """
        if not self.client_loaded:
            self.client = MongoClient(config.MONGODB_URI, uuidRepresentation="standard")
        images = glob.glob(os.path.join(dirname, "*.fits"))

        for image in images:
            self.load_fits(image)



import datetime
import os
import socket
import uuid
import warnings
from typing import Any, Dict, Optional, Type, TypeVar, Union

import astropy.io.fits as fits
import astropy.wcs as awcs
import numpy as np
import qlsc
from geojson_pydantic import Polygon
from pydantic import BaseModel, Field, ValidationError

warnings.simplefilter("ignore", category=awcs.FITSFixedWarning)


Q = qlsc.QLSC(depth=30)
HOSTNAME = socket.gethostname()


EVRImageType = TypeVar("EVRImageType", bound="EVRImage")
EVRImageUpdateType = TypeVar("EVRImageUpdateType", bound="EVRImageUpdate")


def _wcs_to_footprint(header: fits.Header) -> Dict[str, Any]:
    """
     Converts the WCS header information to a GeoJSON polygon footprint.

    Args:
        header: FITS header containing WCS information.

    Returns:
        Dictionary representing the GeoJSON polygon footprint.

    """
    w = awcs.WCS(header)
    im_center = w.all_pix2world(3288, 2192, 0)
    header["CRVAL1"] = float(im_center[0])
    header["CRVAL2"] = float(im_center[1])

    x = np.arange(0, 6575, 274)
    y = np.arange(0, 4384, 274)

    bottom_edge = list(zip([0] * len(x), x)) + [(0, 6575)]
    right_edge = list(zip(y, [6575] * len(y)))[1:] + [(4383, 6575)]
    top_edge = list(zip([4383] * len(x), x)) + [(4383, 6575)]
    top_edge = list(reversed(top_edge))[1:]
    left_edge = list(zip(y, [0] * len(y))) + [(4383, 0)]
    left_edge = list(reversed(left_edge))[1:]

    poly = np.array(bottom_edge + right_edge + top_edge + left_edge)
    poly = np.array([poly.T[1], poly.T[0]]).T

    radecs = w.all_pix2world(poly, 0)
    # Re-orient RA from [-180,180]
    radecs[:, 0] -= 180
    radecs = [list(r) for r in radecs]

    footprint = {"type": "Polygon", "coordinates": [radecs]}
    return footprint


def fitspath_to_constructor(fitspath: Union[str, fits.HDUList]) -> Dict[str, Any]:
    """
    Convert a FITS header into a Pydantic constructor dictionary.

    Args:
        fitspath (str or astropy.io.fits.HDUList): The path to the FITS file or
        and HDUList object.

    Returns:
        Dict[str, Any]: The Pydantic constructor dictionary representing the FITS header.

    Raises:
        KeyError: If any required FITS header fields are missing.
    """
    if type(fitspath) is str:
        hdulist = fits.open(fitspath)

    if len(hdulist) == 1:
        hdu_num = 0
    else:
        hdu_num = 1
    header = hdulist[hdu_num].header.copy()
    hdulist.close()

    if "FIELDID" not in header:
        header["FIELDID"] = 400

    constructor_dict = {}
    constructor_dict["camera"] = header["CCDDETID"]
    constructor_dict["filter_name"] = header["FILTER"]
    constructor_dict["image_type"] = header["IMGTYPE"].strip()
    constructor_dict["obstime"] = header["DATE-OBS"] + "T" + header["TIME-OBS"]
    if "GPSTIME" in header and "Z" in header["GPSTIME"]:
        constructor_dict["gpstime"] = header["GPSTIME"]
    constructor_dict["ccd_set_temp"] = header["SETTEMP"]
    constructor_dict["ccd_temp"] = header["CCDTEMP"]
    constructor_dict["exp_time"] = header["EXPTIME"]
    constructor_dict["site_name"] = header["TELOBS"]
    constructor_dict["server_name"] = HOSTNAME
    constructor_dict["basename"] = header["ORIGNAME"].split(".")[0]
    constructor_dict["fieldid"] = header["FIELDID"]
    constructor_dict["ratchnum"] = header["RATCHNUM"]
    constructor_dict["mount_ha"] = header["MOUNTHA"]
    constructor_dict["sha1"] = header["CHECKSUM"]
    constructor_dict["ccd_ext_temp"] = header["CCDETEMP"]
    constructor_dict["wind_dir"] = header["WINDDIR"]
    constructor_dict["wind_speed"] = header["WINDSPED"]
    constructor_dict["rel_humidity"] = header["OUTRELHU"]
    constructor_dict["dew_point"] = header["OUTDEWPT"]
    constructor_dict["air_pressure"] = header["OUTPRESS"]
    constructor_dict["mushroom_temp"] = header["INR1TEMP"]
    constructor_dict["inc_x"] = header["INR1INCX"]
    constructor_dict["inc_y"] = header["INR1INCY"]
    constructor_dict["inc_z"] = header["INR1INCZ"]

    if "CRVAL1" in header:
        constructor_dict["qid"] = Q.ang2ipix(header["CRVAL1"], header["CRVAL2"])
        constructor_dict["footprint"] = Polygon(**_wcs_to_footprint(header))
        constructor_dict["wcspath"] = os.path.abspath(fitspath)
    else:
        constructor_dict["rawpath"] = os.path.abspath(fitspath)
    return constructor_dict


class EVRImage(BaseModel):
    """Class representing an EVR image."""

    id: str = Field(default_factory=uuid.uuid4, alias="_id")
    camera: Optional[str] = Field(...)
    filter_name: str = Field(...)
    obstime: datetime.datetime = Field(...)
    gpstime: Optional[datetime.datetime] = None
    ccd_set_temp: float = Field(...)
    ccd_temp: float = Field(...)
    exp_time: float = Field(...)

    site_name: str = Field(...)
    server_name: str = Field(...)
    image_type: str = Field(...)
    rawpath: Optional[str] = None
    wcspath: Optional[str] = None
    basename: str = Field(...)
    fieldid: float = Field(...)
    ratchnum: str = Field(...)
    mount_ha: Optional[float] = Field(...)
    sha1: str = Field(...)
    ccd_ext_temp: float = Field(...)
    wind_dir: float = Field(...)
    wind_speed: float = Field(...)
    rel_humidity: float = Field(...)
    dew_point: float = Field(...)
    air_pressure: float = Field(...)
    mushroom_temp: float = Field(...)
    inc_x: Optional[float] = Field(...)
    inc_y: Optional[float] = Field(...)
    inc_z: Optional[float] = Field(...)

    footprint: Optional[Polygon] = Field(...)
    qid: Optional[int] = Field(...)

    class Config:
        """Pydantic configuration."""

        allow_population_by_field_name = True

    @classmethod
    def from_fits(cls: Type[EVRImageType], fitspath: str) -> EVRImageType:
        """Create an instance of EVRImage from a FITS file.

        Args:
            fitspath: Path to the FITS file.

        Returns:
            An instance of EVRImage.

        """

        constructor_dict = fitspath_to_constructor(fitspath)

        # This is duplicated boiler plate between the Image/ImageUpdate classes
        try:
            inst = cls(**constructor_dict)
        except ValidationError as e:
            for error in e.errors():
                field = error["loc"][0]
                constructor_dict[field] = None
            inst = cls(**constructor_dict)

        return inst


class EVRImageUpdate(BaseModel):
    """Class representing an update to an EVR image."""

    camera: Optional[str]
    filter_name: Optional[str]
    obstime: Optional[datetime.datetime]
    gpstime: Optional[datetime.datetime]
    ccd_set_temp: Optional[float]
    ccd_temp: Optional[float]
    exp_time: Optional[float]
    site_name: Optional[str]
    server_name: Optional[str]
    image_type: Optional[str]
    rawpath: Optional[str]
    wcspath: Optional[str]
    basename: Optional[str]
    fieldid: Optional[float]
    ratchnum: Optional[str]
    sha1: Optional[str]
    ccd_ext_temp: Optional[float]
    wind_dir: Optional[float]
    wind_speed: Optional[float]
    rel_humidity: Optional[float]
    dew_point: Optional[float]
    air_pressure: Optional[float]
    mushroom_temp: Optional[float]
    inc_x: Optional[float]
    inc_y: Optional[float]
    inc_z: Optional[float]
    footprint: Optional[Polygon]
    qid: Optional[int]

    class Config:
        """Pydantic configuration."""

        allow_population_by_field_name = True

    @classmethod
    def from_fits(cls: Type[EVRImageUpdateType], fitspath: str) -> EVRImageUpdateType:
        """Create an instance of EVRImageUpdate from a FITS file.

        Args:
            fitspath: Path to the FITS file.

        Returns:
            An instance of EVRImageUpdate.

        """
        constructor_dict = fitspath_to_constructor(fitspath)

        # This is duplicated boiler plate between the Image/ImageUpdate classes
        try:
            inst = cls(**constructor_dict)
        except ValidationError as e:
            for error in e.errors():
                field = error["loc"][0]
                constructor_dict[field] = None
            inst = cls(**constructor_dict)

        return inst



"""Top-level package for Evryscope-Argus Transient Reporter."""

__author__ = """Hank Corbett"""
__email__ = "htc@unc.edu"
__version__ = "0.0.2"

__all__ = ["RicoHeartBeat", "RawStreamer", "EVRImageLoader"]

import logging
import os

from dotenv import load_dotenv

basedir = os.path.abspath(os.path.dirname(__file__))
load_dotenv(os.path.join(basedir, ".env"))


class Config(object):
    """
    Configuration class for managing application settings.

    Attributes:
        LOCAL_ADDR (str): The local address to bind the application to. Defaults to "127.0.0.1" if not provided in the environment.
        KAFKA_ADDR (str): The address of the Kafka server. Defaults to "152.2.38.172" if not provided in the environment.
        KAFKA_PORT (str): The port of the Kafka server. Defaults to "9092" if not provided in the environment.
        HBEAT_TOPIC (str): The topic name for heartbeat messages in Kafka. Defaults to "rico-hearbeat" if not provided in the environment.
    """

    SLACK_SIGNING_SECRET = os.environ.get("SLACK_SIGNING_SECRET") or None
    SLACK_BOT_TOKEN = os.environ.get("SLACK_BOT_TOKEN") or None
    SLACK_PORT = os.environ.get("SLACK_PORT") or 3000

    LOCAL_ADDR = os.environ.get("LOCAL_ADDR") or "127.0.0.1"

    KAFKA_ADDR = os.environ.get("KAFKA_ADDR") or "127.0.0.1"
    KAFKA_PORT = os.environ.get("KAFKA_PORT") or "9092"
    HBEAT_TOPIC = os.environ.get("HBEAT_TOPIC") or "rico.heartbeat"
    RAW_TOPIC_BASE = os.environ.get("RAW_TOPIC_BASE") or "rico.candidates.raw"
    EVR_IMAGE_TOPIC = os.environ.get("RAW_TOPIC_BASE") or "rico.images.evr"

    MONGODB_URI = os.environ.get("MONGODB_URI") or None
    MONGO_DBNAME = os.environ.get("MONGO_DBNAME") or "hdps"


config = Config()


def get_logger(name: str) -> logging.Logger:
    """
    Shortcut to grab a logger instance with the specified name.

    Args:
        name (str): The name of the logger.

    Returns:
        logging.Logger: A logger instance configured with the specified name.
    """

    logger = logging.getLogger(name)
    if not logger.handlers:
        # Prevent logging from propagating to the root logger
        logger.propagate = False
        logger.setLevel(("INFO"))
        console = logging.StreamHandler()
        logger.addHandler(console)
        formatter = logging.Formatter(
            "%(asctime)s — %(name)s — %(levelname)s — %(funcName)s:%(lineno)d — %(message)s"
        )
        console.setFormatter(formatter)
    return logger


# For testing
get_logger(__name__).addHandler(logging.StreamHandler())

# Silenced
# get_logger(__name__).addHandler(logging.NullHandler())


from .heartbeat import RicoHeartBeat  # noqa: E402
from .images import EVRImageLoader  # noqa: E402
from .raw_streamer import RawStreamer  # noqa: E402



__all__ = [
    "EVRImage",
    "EVRImageUpdate",
    "EVRImageType",
    "EVRImageUpdateType",
    "fitspath_to_constructor",
]

from .evr_image import (
    EVRImage,
    EVRImageType,
    EVRImageUpdate,
    EVRImageUpdateType,
    fitspath_to_constructor,
)



import subprocess
import threading
from typing import Optional


class EFTERunner(threading.Thread):
    """A class representing a thread that runs a command using subprocess."""

    def __init__(self) -> None:
        """Initialize the EFTERunner thread."""
        self.stdout: Optional[bytes] = None
        self.stderr: Optional[bytes] = None
        threading.Thread.__init__(self)

    def run(self, command: str) -> None:
        """Execute the specified command in a separate thread.

        Args:
            command (str): The command to be executed.

        Returns:
            None.
        """
        p = subprocess.Popen(
            command.split(), shell=False, stdout=subprocess.PIPE, stderr=subprocess.PIPE
        )

        self.stdout, self.stderr = p.communicate()



__all__ = ["RicoHeartBeat"]

import datetime
import threading
from typing import Any, Callable, Dict, Optional, Tuple

from . import config
from .producer import Producer


class RepeatTimer(threading.Timer):
    finished: threading.Event
    interval: float
    function: Callable[..., Any]
    args: Tuple[Any, ...]
    kwargs: Dict[str, Any]

    def run(self) -> None:
        while not self.finished.wait(self.interval):
            assert self.function is not None
            assert self.args is not None
            assert self.kwargs is not None
            self.function(*self.args, **self.kwargs)


class RicoHeartBeat(Producer):
    def __init__(self) -> None:
        """
        Initialize the RicoHeartBeat instance.

        It inherits from the Producer class and configures the Kafka connection parameters
        using values from the config dictionary.

        The host, port, and topic are retrieved from the config dictionary
        using the keys 'KAFKA_ADDR', 'KAFKA_PORT', and 'HBEAT_TOPIC' respectively.
        """
        super().__init__(
            host=config.KAFKA_ADDR,
            port=config.KAFKA_PORT,
        )
        self.heartbeat_thread: Optional[RepeatTimer] = None

    def send_heartbeat(self) -> None:
        """
        Send a heartbeat message to the Kafka topic.

        The message is a dictionary containing the current UTC time.
        """
        self.send_json({"utc": datetime.datetime.utcnow()}, config.HBEAT_TOPIC)

    def start(self, rate: int = 30) -> None:
        """
        Start sending heartbeat messages periodically.

        Args:
            rate (int): The rate at which heartbeat messages should be sent, in seconds.
                        Defaults to 30 seconds.
        """
        self.heartbeat_thread = RepeatTimer(rate, self.send_heartbeat)
        self.heartbeat_thread.start()

    def stop(self) -> None:
        """
        Stop sending heartbeat messages periodically.
        """
        if self.heartbeat_thread is not None:
            self.heartbeat_thread.cancel()



__all__ = ["RawStreamer"]

import io
import os
from typing import Any, Dict, Tuple

import astropy.table as tbl
import blosc
import fastavro as fa
import orjson

from . import config
from .producer import Producer

PATH = os.path.realpath(os.path.dirname(__file__))


class RawStreamer(Producer):
    def __init__(self) -> None:
        """
        Initialize the RawStreamer instance.

        It inherits from the Producer class and configures the Kafka connection parameters
        using values from the config dictionary.

        This class is used for streaming raw candidate detections from the
        observatory to the Rico Kafka cluster.

        The host, port, and topic are retrieved from the config dictionary
        using the keys 'KAFKA_ADDR', 'KAFKA_PORT', and 'HBEAT_TOPIC' respectively.
        """
        super().__init__(
            host=config.KAFKA_ADDR,
            port=config.KAFKA_PORT,
        )
        self.topic_base = config.RAW_TOPIC_BASE

    def _parse_raw_catalog(self, catalog_path: str) -> Tuple[bytes, Dict[str, Any]]:
        """
        Parses a raw catalog file and returns the serialized Avro data.

        Args:
            catalog_path (str): The path to the catalog file.

        Returns:
            bytes: The serialized Avro data.
            dict: Catalog metadata.
        """
        tab = tbl.Table.read(catalog_path)

        mjd = tab.meta["MJD"]
        camera_id = tab.meta["CCDDETID"]

        records = []
        for r in tab:
            stamp = blosc.compress(r["stamp"].tobytes())
            record = dict(r)
            record["stamp"] = stamp
            record["epoch"] = mjd
            record["camera"] = camera_id

            records.append(record)

        with open(f"{PATH}/schemas/raw_candidate.avsc", "rb") as f:
            schema = orjson.loads(f.read())

        parsed_schema = fa.parse_schema(schema)

        fo = io.BytesIO()
        fa.writer(fo, parsed_schema, records)
        fo.seek(0)

        return fo.read(), tab.meta

    def push_from_catalog(self, catalog_path: str) -> None:
        """
        Pushes data from a catalog file to a camera-specific topic.

        Args:
            catalog_path (str): The path to the catalog file.

        Returns:
            None

        """
        avro_data, metadata = self._parse_raw_catalog(catalog_path)
        topic = self.topic_base + f'.{metadata["CCDDETID"]}'

        self.send_binary(avro_data, topic=topic)



import sys
from typing import Sequence

import click


@click.group()
def main() -> None:
    """Console scripts for rico."""
    pass


class Config(object):
    """Contains global options used for all pipeline actions."""

    def __init__(self) -> None:
        self.verbose = False
        self.ncpus = 72


pass_config = click.make_pass_decorator(Config, ensure=True)


if __name__ == "__main__":
    sys.exit(main())  # pragma: no cover


@click.command("slack", short_help="Start Rico Slack server.")
def slack() -> None:
    """Starts the Rico Slack server."""
    from .slack import slack_app_starter

    slack_app_starter()


@click.command("index_images", short_help="Index EVR images directly into MongoDB.")
@click.argument("directories", nargs=-1, type=click.Path(exists=True))
@pass_config
def index_images(cli_config: Config, directories: Sequence[str]) -> None:
    """Indexes EVR images from the specified directories.

    Args:
        cli_config: Config object containing global options.
        directories: Iterable of directories containing EVR images.

    """
    import multiprocessing as mp

    from . import EVRImageLoader

    image_loader = EVRImageLoader(create_client=False)

    pool = mp.Pool(cli_config.ncpus)

    directories = sorted(directories)
    n_directories = len(directories)
    out = []
    with click.progressbar(
        pool.imap_unordered(image_loader.load_directory, directories),
        label="Loading: ",
        length=n_directories,
    ) as pbar:
        for _ in pbar:
            out.append(_)

    pool.close()
    pool.join()


main.add_command(slack)
main.add_command(index_images)



__all__ = ["slack_app", "slack_app_starter"]
import datetime
import os
import time
from typing import Any, Callable, Dict

import slack_sdk as sk
from sanitize_filename import sanitize
from slack_bolt import App

from . import config, efte_runner, get_logger
from . import images as rimages

log = get_logger("slack_bot")

slack_app = App(
    token=config.SLACK_BOT_TOKEN,
    signing_secret=config.SLACK_SIGNING_SECRET,
)


@slack_app.event("message")
def handle_message_events(body: Dict[str, Any]) -> None:
    """Handle message events in the Slack app.

    Args:
        body (Dict[str, Any]): The payload containing the event data.

    Returns:
        None.
    """
    pass


@slack_app.message("hi Rico")
def message_hello(message: Dict[str, Any], say: Callable[..., None]) -> None:
    """Handle the "hi Rico" message event in the Slack app.

    Args:
        message (Dict[str, Any]): The incoming message data.
        say (Callable[..., None]): Function to send a message as a response.

    Returns:
        None.
    """
    say(f"Hey there <@{message['user']}>!", thread_ts=message["ts"])


@slack_app.event("reaction_added")
def telescope_trigger(
    body: Dict[str, Any], ack: Callable[..., None], say: Callable[..., None]
) -> None:
    """
    Triggered when a reaction is added in the Slack app.

    Args:
        body (dict): The payload containing the event data.
        ack (function): Acknowledgment function to confirm the event was received.
        say (function): Function to send a message as a response.

    Returns:
        None.
    """
    # Acknowledge the action
    ack()
    if "telescope" in body["event"]["reaction"]:
        event = body["event"]
        thread_ts = event["item"]["ts"]
        say(
            f"<@{event['user']}> initiated a follow-up request.",
            thread_ts=thread_ts,
        )


@slack_app.shortcut("rico_lightcurve_req")
def open_lcv_modal(ack, shortcut, client):
    """Open a modal for requesting a lightcurve in the Slack app.

    Args:
        ack (Callable[..., None]): Acknowledgment function to confirm the shortcut request.
        shortcut (Dict[str, Any]): The shortcut data.
        client: The Slack WebClient instance.

    Returns:
        None.
    """
    # Acknowledge the shortcut request
    ack()
    # Call the views_open method using the built-in WebClient
    client.views_open(
        trigger_id=shortcut["trigger_id"],
        # A simple view payload for a modal
        view={
            "type": "modal",
            "callback_id": "modal-lcv-req",
            "title": {"type": "plain_text", "text": "Forced Photometry", "emoji": True},
            "submit": {"type": "plain_text", "text": "Submit", "emoji": True},
            "close": {"type": "plain_text", "text": "Cancel", "emoji": True},
            "blocks": [
                {
                    "type": "input",
                    "block_id": "target_name",
                    "element": {
                        "type": "plain_text_input",
                        "action_id": "title",
                        "placeholder": {"type": "plain_text", "text": "TRAPPIST-1"},
                    },
                    "label": {"type": "plain_text", "text": "Target Name"},
                },
                # ... rest of the blocks ...
            ],
        },
    )


@slack_app.view("modal-lcv-req")
def handle_submission(
    ack: Callable[..., None], body: Dict[str, Any], client: sk.WebClient
) -> None:
    """Handle the submission of the lightcurve request modal in the Slack app.

    Args:
        ack (Callable[..., None]): Acknowledgment function to confirm the submission.
        body (Dict[str, Any]): The payload containing the submission data.
        client: The Slack WebClient instance.

    Returns:
        None.
    """
    values = body["view"]["state"]["values"]
    user = body["user"]["id"]

    target_name = sanitize(values["target_name"]["title"]["value"])
    start_date = values["start_date"]["datepicker-action"]["selected_date"]
    end_date = values["end_date"]["datepicker-action"]["selected_date"]
    right_ascension = float(values["right_ascension"]["number_input-action"]["value"])
    declination = float(values["declination"]["number_input-action"]["value"])

    errors = {}
    if (right_ascension < 0) or (right_ascension > 360):
        errors["right_ascension"] = "Right ascension out of range."
    if (declination < -90) or (declination > 90):
        errors["declination"] = "Declination out of range."
    if len(errors) > 0:
        ack(response_action="errors", errors=errors)
        return
    # Acknowledge the view_submission request and close the modal
    ack()
    # Do whatever you want with the input data - here we're querying the DB and
    # then passing off the lcv gen to a background worker

    # TODO: probably can refactor this to only hit the DB once, but what's 15
    # seconds out of 5 hours?
    nimages = len(
        rimages.images_containing(
            right_ascension,
            declination,
            datetime.datetime.strptime(start_date, "%Y-%m-%d"),
            datetime.datetime.strptime(end_date, "%Y-%m-%d"),
        )
    )
    uncompleteable = False
    # Message to send user
    msg = ""
    try:
        if nimages >= 1:
            msg = f"Hi <@{user}>! I have received your request to produce the following lightcurve:\n*{target_name}* at RA: {right_ascension}, Dec: {declination}.\nFrom {start_date} to {end_date}.\n\n *Please note that I don't know how long this will take! Your table will be uploaded here ASAP.*"
        else:
            msg = f"Hi <@{user}>! I received your forced photometry request, but didn't find any images."
            uncompleteable = True
    except Exception as e:
        # Handle error
        msg = f"There was an error with your submission: {e}"
    #
    # Message the user
    try:
        response = client.chat_postMessage(channel=user, text=msg)
    except Exception as e:
        print(f"Failed to post a message {e}")
        return
    if uncompleteable:
        return

    dm_channel = response["channel"]

    er = efte_runner.EFTERunner()
    er.run(
        f"efte -n 72 autophot --mindate {start_date} --maxdate {end_date} -o {target_name} {right_ascension} -- {declination}"
    )
    output = (
        target_name
        + f"_{right_ascension:.2f}_{declination:.2f}".replace(".", "d").replace(
            "-", "m"
        )
        + ".fits"
    )
    while not os.path.isfile(output):
        time.sleep(60)

    client.files_upload_v2(
        file=f"./{output}",
        channel=dm_channel,
        title=output,
        initial_comment="Your lightcurve is ready!",
    )


def slack_app_starter() -> None:
    log.info("Slack integration starting up...")
    slack_app.start(port=int(config.SLACK_PORT))



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['argus_rico', 'argus_rico.models']

package_data = \
{'': ['*'], 'argus_rico': ['schemas/*']}

install_requires = \
['astropy>=5.3,<6.0',
 'black>=23.3.0,<24.0.0',
 'blosc>=1.11.1,<2.0.0',
 'click>=8.1.3,<9.0.0',
 'confluent-kafka>=2.1.1,<3.0.0',
 'fastapi>=0.95.2,<0.96.0',
 'fastavro>=1.4.12,<2.0.0',
 'geojson-pydantic>=0.6.2,<0.7.0',
 'ipython>=8.14.0,<9.0.0',
 'numpy>=1.24.3,<2.0.0',
 'orjson>=3.8.13,<4.0.0',
 'pyarrow>=10.0.0',
 'pydantic>=1.10.8,<2.0.0',
 'pymongo>=4.3.3,<5.0.0',
 'pymongoarrow>=0.7.0,<0.8.0',
 'python-dotenv>=0.19.0,<0.20.0',
 'rich>=13.3.5,<14.0.0',
 'sanitize-filename>=1.2.0,<2.0.0',
 'slack-bolt>=1.18.0,<2.0.0']

entry_points = \
{'console_scripts': ['rico = rico.cli:main']}

setup_kwargs = {
    'name': 'argus-rico',
    'version': '0.0.2',
    'description': 'Transient alert generation and database interaction for Argus and Evryscope',
    'long_description': None,
    'author': 'Hank Corbett',
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.10,<3.11',
}


setup(**setup_kwargs)



from typing import Any, Dict, Optional, Union
from uuid import uuid4

import confluent_kafka as ck
import orjson

from . import get_logger


class Consumer:
    def __init__(
        self,
        host: str,
        port: Union[int, str],
        group_id: str,
    ) -> None:
        self.c = ck.Consumer(
            {
                "bootstrap.servers": f"{host}:{port}",
                "group.id": group_id,
                "enable.auto.commit": False,
                "auto.offset.reset": "earliest",
            }
        )
        self.log = get_logger(__name__)
